
#include <plib.h>
#include "stdtypes.h"
#include "config.h"
#include "MtrCtrl.h"
#include "spi.h"
#include "util.h"


#ifndef OVERRIDE_CONFIG_BITS

#pragma config ICESEL   = ICS_PGx2		// ICE/ICD Comm Channel Select
#pragma config BWP      = OFF			// Boot Flash Write Protect
#pragma config CP       = OFF			// Code Protect
#pragma config FNOSC    = PRIPLL		// Oscillator Selection
#pragma config FSOSCEN  = OFF			// Secondary Oscillator Enable
#pragma config IESO     = OFF			// Internal/External Switch-over
#pragma config POSCMOD  = HS			// Primary Oscillator
#pragma config OSCIOFNC = OFF			// CLKO Enable
#pragma config FPBDIV   = DIV_8			// Peripheral Clock divisor
#pragma config FCKSM    = CSDCMD		// Clock Switching & Fail Safe Clock Monitor
#pragma config WDTPS    = PS1			// Watchdog Timer Postscale
#pragma config FWDTEN   = OFF			// Watchdog Timer 
#pragma config FPLLIDIV = DIV_2			// PLL Input Divider
#pragma config FPLLMUL  = MUL_20		// PLL Multiplier
#pragma config UPLLIDIV = DIV_2			// USB PLL Input Divider
#pragma config UPLLEN   = OFF			// USB PLL Enabled
#pragma config FPLLODIV = DIV_1			// PLL Output Divider
#pragma config PWP      = OFF			// Program Flash Write Protect
#pragma config DEBUG    = OFF			// Debugger Enable/Disable


#endif


#define		TCKPS22 			6
#define 	TCKPS21				5
#define 	TCKPS20				4
#define		TCKPS32 			6
#define 	TCKPS31				5
#define 	TCKPS30				4


#define I2C_CLOCK_FREQ     100000
#define period 	10

#define	stPressed	1
#define	stReleased	0

struct btn {
	BYTE	stBtn;	// status of the button (pressed or released)
	BYTE	stCur;  // current read state of the button
	BYTE	stPrev; // previous read state of the button
	BYTE	cst;	// number of consecutive reads of the same button 
					// state
};


volatile	struct btn	btnBtn1;
volatile	struct btn	btnBtn2;

volatile	struct btn	PmodBtn1;
volatile	struct btn	PmodBtn2;
volatile	struct btn	PmodBtn3;
volatile	struct btn	PmodBtn4;

volatile	struct btn	PmodSwt1;
volatile	struct btn	PmodSwt2;
volatile	struct btn	PmodSwt3;
volatile	struct btn	PmodSwt4;

BYTE	min1=0;
BYTE	min2=0;
BYTE	hr1=0;
BYTE	hr2=0;
BYTE 	aux1=0;
BYTE	aux3=0;
BYTE	fullmin=0;
BYTE	fullhr=0;
BOOL on=fTrue;


unsigned int x,aux=0, once=0, trigger=0,servo=1;



void	DeviceInit(void);
void	Wait_ms(WORD ms);



void __ISR(_TIMER_4_VECTOR, ipl7) _Timer4Handler(void)
{ 


	//vary the frequency
	if(trigger)
	{
	x++;
	if (x==1000)
	{ 
 		if((PR4<10000)&&(aux==0)) PR4+=500;
		else
		{
			aux=1;
			if((PR4>10)&&(aux==1)) PR4-=1000;
			else aux=0;
		}
		x=0;
		
	}
	}
	mT4ClearIntFlag(); 
}

	  

void __ISR(_TIMER_2_VECTOR, ipl7) _Timer2Handler(void) {

	if(servo)
{
	StartServoPulse();
	
	UpdateServoPos();
}	

	IFS0CLR	= ( 1 << 8 ); 	// clear interrupt flag for timer 2
}

void __ISR(_OUTPUT_COMPARE_1_VECTOR, ipl7) OC1_IntHandler (void) {

	
if(servo)	EndServoPulse();
	

	IFS0CLR	= ( 1 << 6 );	// clear interrupt flag for output compare 1	
}



void __ISR(_TIMER_5_VECTOR, ipl7) Timer5Handler(void)
{
	unsigned int i;
	mT5ClearIntFlag();

		if(trigger)// play the alarm at the right time
	{
	
		if(TMR4<(PR4*0.75)) PORTF|=(1<<12);
		else PORTF&=~(1<<12);
	}
		


	 
	// Read the raw state of the button pins.
	btnBtn1.stCur = ( prtBtn1 & ( 1 << bnBtn1 ) ) ? stPressed : stReleased;
	btnBtn2.stCur = ( prtBtn2 & ( 1 << bnBtn2 ) ) ? stPressed : stReleased;
	
	//Read the raw state of the PmodBTN pins
	PmodBtn1.stCur = ( prtJE1 & ( 1 << bnJE1 ) ) ? stPressed : stReleased;
	PmodBtn2.stCur = ( prtJE2 & ( 1 << bnJE2 ) ) ? stPressed : stReleased;
	PmodBtn3.stCur = ( prtJE3 & ( 1 << bnJE3 ) ) ? stPressed : stReleased;
	PmodBtn4.stCur = ( prtJE4 & ( 1 << bnJE4 ) ) ? stPressed : stReleased;

	//Read the raw state of the PmodSWT pins
	PmodSwt1.stCur = ( prtJA1 & ( 1 << swtJA1 ) ) ? stPressed : stReleased;
	PmodSwt2.stCur = ( prtJA2 & ( 1 << swtJA2 ) ) ? stPressed : stReleased;
//	PmodSwt3.stCur = ( prtJA3 & ( 1 << swtJA3 ) ) ? stPressed : stReleased;
//	PmodSwt4.stCur = ( prtJA4 & ( 1 << swtJA4 ) ) ? stPressed : stReleased;

	// Update state counts.
	btnBtn1.cst = ( btnBtn1.stCur == btnBtn1.stPrev ) ? btnBtn1.cst + 1 : 0;
	btnBtn2.cst = ( btnBtn2.stCur == btnBtn2.stPrev ) ? btnBtn2.cst + 1 : 0;

	//Update state counts for PmodBTN
	PmodBtn1.cst = (PmodBtn1.stCur == PmodBtn1.stPrev) ? PmodBtn1.cst +1 : 0;
	PmodBtn2.cst = (PmodBtn2.stCur == PmodBtn2.stPrev) ? PmodBtn2.cst +1 : 0;
	PmodBtn3.cst = (PmodBtn3.stCur == PmodBtn3.stPrev) ? PmodBtn3.cst +1 : 0;
	PmodBtn4.cst = (PmodBtn4.stCur == PmodBtn4.stPrev) ? PmodBtn4.cst +1 : 0;

	//Update state counts for PmodSWT
	PmodSwt1.cst = (PmodSwt1.stCur == PmodSwt1.stPrev) ? PmodSwt1.cst +1 : 0;
	PmodSwt2.cst = (PmodSwt2.stCur == PmodSwt2.stPrev) ? PmodSwt2.cst +1 : 0;
//	PmodSwt3.cst = (PmodSwt3.stCur == PmodSwt3.stPrev) ? PmodSwt3.cst +1 : 0;
//	PmodSwt4.cst = (PmodSwt4.stCur == PmodSwt4.stPrev) ? PmodSwt4.cst +1 : 0;
	
	// Save the current state.
	btnBtn1.stPrev = btnBtn1.stCur;
	btnBtn2.stPrev = btnBtn2.stCur;

	// Save the current state for PmodBTN
	PmodBtn1.stPrev = PmodBtn1.stCur;
	PmodBtn2.stPrev = PmodBtn2.stCur;
	PmodBtn3.stPrev = PmodBtn3.stCur;
	PmodBtn4.stPrev = PmodBtn4.stCur;

	// Save the current state for PmodSWT
	PmodSwt1.stPrev = PmodSwt1.stCur;
	PmodSwt2.stPrev = PmodSwt2.stCur;
	PmodSwt3.stPrev = PmodSwt3.stCur;
	PmodSwt4.stPrev = PmodSwt4.stCur;
	
	// Update the state of button 1 

	btnBtn1.stBtn = btnBtn1.stCur;

	
	// Update the state of button 2 

		btnBtn2.stBtn = btnBtn2.stCur;


	//if statements for buttons

	// Update the state of PmodBTN1 

	PmodBtn1.stBtn = PmodBtn1.stCur;

	
	// Update the state of PmodBTN2 

	PmodBtn2.stBtn = PmodBtn2.stCur;


	// Update the state of PmodBTN3 

	PmodBtn3.stBtn = PmodBtn3.stCur;


	// Update the state of PmodBTN4 

	PmodBtn4.stBtn = PmodBtn4.stCur;


	//if statements for switches

	// Update the state of PmodSWT1 

	PmodSwt1.stBtn = PmodSwt1.stCur;

	
	// Update the state of PmodSWT2 

	PmodSwt2.stBtn = PmodSwt2.stCur;


	// Update the state of PmodSWT3 

//	PmodSwt3.stBtn = PmodSwt3.stCur;


	// Update the state of PmodSWT4 

//	PmodSwt4.stBtn = PmodSwt4.stCur;


}



int main(void) {

	BYTE	stBtn1;
	BYTE	stBtn2;

	BYTE	stPmodBtn1;
	BYTE	stPmodBtn2;
	BYTE	stPmodBtn3;
	BYTE	stPmodBtn4;

	BYTE	stPmodSwt1;
	BYTE	stPmodSwt2;
	BYTE	stPmodSwt3;
	BYTE	stPmodSwt4;

	BYTE first=1;

	BOOL firsttime=fTrue;
	BOOL firstalarm=fTrue;
	BOOL busy=fTrue;
	
	BOOL go=fFalse;
	BOOL once=fTrue;;	

	OpenI2C1( I2C_EN, I2C_CLOCK_FREQ  ); //enable I2C transmission, at the frequency BRG_VAL


	DeviceInit();// init the timers and the DC motors
	initADC(); // init the adc module

	SpiClearScreen();	
	SpiPutBuff("Welcome",7);
	DelayMs(2000);// Welcome message

	SpiClearScreen();
	SpiPutBuff("Enter the time",14);
	DelayMs(2000);

	// display 00:00, at the very first beginning					

	Display_time(min1,min2,hr1,hr2);

	INTEnableInterrupts();

	while (fTrue)
{
	

		if(go) //if the alarm has begun
	{
			while((Get0()>=0x250)&&(Get0()>=0x250)&&(Get0()>=0x250))//as long as somebody tries to stop the alarm
			{
				servo=0;//stop the radar
				trigger=0;//stop the alarm tune
		
			if((Get1()<=0x150)&&(Get2()<=0x150))//if both left and right sides are claer, go forward
			{
			MtrCtrlStop();  //stop before sending new data to avoid possible short circuit
			UpdateMotors(); 
			Wait_ms(0x0A00);
			MtrCtrlFwd();
			UpdateMotors();
		while( ((Get1()<=0x150)&&(Get2()<=0x150))&&(Get0()>=0x200)) {DelayMs(0x0400);}
			MtrCtrlStop();
			UpdateMotors();	
			
			}
			
			
		else if ((Get1()>=0x150)&&(Get2()<=0x150))//if an obstacle is detected on the left side, go back and turn left
		{
			MtrCtrlStop();  //stop before sending new data to avoid possible short circuit
			UpdateMotors(); 
			Wait_ms(0x0A00);
			MtrCtrlBwd();
			UpdateMotors();
		while (((Get1()>=0x150)&&(Get0()>=0x200))){DelayMs(0x0200);}
			MtrCtrlStop();
			UpdateMotors();
			
			MtrCtrlStop();  //stop before sending new data to avoid possible short circuit
			UpdateMotors(); 
			Wait_ms(0x0A00);
			MtrCtrlLeft();
			UpdateMotors();
			DelayMs(0x0300);
			MtrCtrlStop();
			UpdateMotors();
		}

		else if ((Get2()>=0x150)&&(Get1()<=0x150))//if an obstacle is detected on the right side, go back and turn right
		{
			MtrCtrlStop();  //stop before sending new data to avoid possible short circuit
			UpdateMotors(); 
			Wait_ms(0x0A00);
			MtrCtrlBwd();
			UpdateMotors();
		while (((Get2()>=0x150)&&(Get0()>=0x200))){DelayMs(0x0200);}
			MtrCtrlStop();
			UpdateMotors();		

			MtrCtrlStop();  //stop before sending new data to avoid possible short circuit
			UpdateMotors(); 
			Wait_ms(0x0A00);
			MtrCtrlRight();
			UpdateMotors();
			DelayMs(0x0300);
			MtrCtrlStop();
			UpdateMotors();
		}

		else if ((Get2()>=0x150)&&(Get1()>=0x150))//if both front sensors see an obstacle, go back and evaluate the position
		{
			MtrCtrlStop();  //stop before sending new data to avoid possible short circuit
			UpdateMotors(); 
			Wait_ms(0x0A00);
			MtrCtrlBwd();
			UpdateMotors();
		while ((((Get2()>=0x150)||(Get1()>0x150))&&(Get0()>=0x200))){DelayMs(0x0200);}
			MtrCtrlStop();
			UpdateMotors();

			if(Get1()>=Get2())// if the obstacle is closer to the left side, turn left
			{

			MtrCtrlStop();  //stop before sending new data to avoid possible short circuit
			UpdateMotors(); 
			Wait_ms(0x0A00);
			MtrCtrlLeft();
			UpdateMotors();
			DelayMs(0x0400);
			MtrCtrlStop();
			UpdateMotors();
		
			}

			else//otherwise turn right
			{
			MtrCtrlStop();  //stop before sending new data to avoid possible short circuit
			UpdateMotors(); 
			Wait_ms(0x0A00);
			MtrCtrlRight();
			UpdateMotors();
			DelayMs(0x0400); 
			MtrCtrlStop();
			UpdateMotors();
			}
		}
			}
		if(on)
		{
		//as long as the sensor on radar doesn't see anything, turn the radar and the alarm tune on
		servo=1;//
		trigger=1;
		}
}
	  

		//get the data from the buttons and switches
		stBtn1 = btnBtn1.stBtn;
		stBtn2 = btnBtn2.stBtn;

		stPmodBtn1 = PmodBtn1.stBtn;
		stPmodBtn2 = PmodBtn2.stBtn;
		stPmodBtn3 = PmodBtn3.stBtn;
		stPmodBtn4 = PmodBtn4.stBtn;

		stPmodSwt1 = PmodSwt1.stBtn;
		stPmodSwt2 = PmodSwt2.stBtn;
//		stPmodSwt3 = PmodSwt3.stBtn;
//		stPmodSwt4 = PmodSwt4.stBtn;

	if (stPressed == stBtn1) 
	{
		trigger=0;//reset the control variables so that the alarm may be triggered again
		servo=0;
		on=fFalse;
		go=fFalse;
	}	

	if(stPressed== stBtn2) on=fTrue;

		if(stPressed == stPmodBtn1){
		//set minutes
		
		busy=fTrue;// while setting the time, the screen must remain steady
	
		if(min1<9) min1++;
		else
		{
			min1=0;
			if(min2<5) min2++;
			else min2=0;
		}
			// display the updated time
			Display_time(min1,min2,hr1,hr2);

		}// end PmodBtn1


		else if (stPressed == stPmodBtn2) {
		//set hour 
	
		busy=fTrue;

		if(((hr1<9)&&(hr2<=1))||((hr1<3)&&(hr2==2))) hr1++;
		else
		{
			if((hr1==9)&& (hr2<=1))
			{
				hr1=0;
				hr2++;
			}
			else 
			{
				hr1=0;
				hr2=0;
			}
		}
			// display the updated time
			Display_time(min1,min2,hr1,hr2);			
						
	}// end PmodBtn2


		else if (stPressed == stPmodBtn3){
		// save time/alarm
	
		busy=fTrue;

		if( stPressed== stPmodSwt1 )
		{				
			//save the time
			aux1=0;
			aux1=min2;
			aux1=aux1<<4;
			aux1|=min1;
						
			Write(aux1,0x01);//save the minutes
						
			aux1=0;
			aux1=hr2;
			aux1=aux1<<4;
			aux1|=hr1;

			Write(aux1,0x02);//save the hours						

			if(firsttime) 
			{
				Write(0x80,0x00);
				firsttime=fFalse;// if it is the first hour set, the RTCC module must be turned on
			}
					
			SpiClearScreen();
			SpiPutBuff("Time saved",10);
			DelayMs(2000);
					

					
			//display the updated time
			Display_time(min1,min2,hr1,hr2);
						
		}


		else
		{
			//save the alarm
			aux1=0;
			aux1=min2;
			aux1=aux1<<4;
			aux1|=min1;						

			Write(aux1,0x0B);//save the alarm minutes

			aux1=0;
			aux1=hr2;
			aux1=aux1<<4;
			aux1|=hr1;
			Write(aux1,0x0C);// save the alarm hours

			SpiClearScreen();
			SpiPutBuff("Alarm saved",11);
			DelayMs(2000);
			
			aux3=1;

			if(firstalarm)
			{
				Write(0x10,0x07);// if it is the first alarm set, the alarm register must be activated
				firstalarm=fFalse;
			}

			//display the updated time
			Display_time(min1,min2,hr1,hr2);
						 

		}
	busy=fFalse;//the time/alarm setting is finished
	}// end PmodBtn3


			 
		else if (stPressed == stPmodBtn4){

		busy=fTrue;

		if( stPressed== stPmodSwt1 )
		{	
						
			SpiClearScreen();
			SpiPutBuff("Enter the time",14);
			DelayMs(2000);
			
			//display the updated time
			Display_time(min1,min2,hr1,hr2);				
		}
 
		else
		{
			SpiClearScreen();
			SpiPutBuff("Enter the alarm",15);
			DelayMs(2000);

			//display the updated time			
			Display_time(min1,min2,hr1,hr2);
							
		}

	busy=fFalse;

						
	}// end PmodBtn4

	
		
if(!busy)
{	

	if((Read(0x01)&0x0F)!=min1)//if the unit minutes have changed, display the updated time
	{
		min1=(Read(0x01)&0x0F);
		Display_time(min1,min2,hr1,hr2);
	}
	
		if(((Read(0x01)&0xF0)>>4)!=min2)//if the tens minutes have changed, display the updated time
	{
		min2=(Read(0x01)&0xF0);
		min2=min2>>4;
		Display_time(min1,min2,hr1,hr2);
	}

	if((Read(0x02)&0x0F)!=hr1)// if the unit hours have changed, display the updated time
	{
		hr1=(Read(0x02)&0x0F);
		Display_time(min1,min2,hr1,hr2);
	}

	if(((Read(0x02)&0xF0)>>4)!=hr2)// if the tens hours have changed, display the updated time
	{
		hr2=(Read(0x02)&0xF0);
		hr2=hr2>>4;
		Display_time(min1,min2,hr1,hr2);
	}



	if((Read(0x01)==Read(0x0B))&&(Read(0x02)==Read(0x0C))&&(firsttime==fFalse)&&(firstalarm==fFalse))// if it's alarm time, ring the alarm and display a proper message
	{
		
			if(on)
		{
				SpiClearScreen();
				SpiPutBuff("ALARM!!!",8);
				DelayMs(2000);
				go=fTrue;
				trigger = 1;		
			//if the alarm has been triggered just now
				
			if(once)
			{
				ServoInit();//init the servo motor
				T4CON	= ( 1 << 5 ) | ( 1 << 4 ) | (1 << 15);//start playing the alarm song
				once=fFalse;
			}		
							
		}

}
	}	
}
}
	


void DeviceInit() {

	TMR4 	= 0; //clear timer 4 count
	PR4		= period; //set the rate of the timer interrupt
	//Set up timer4 interrupt
	IPC4SET	= ( 1 << 4 ) | ( 1 << 3 ) | ( 1 << 2 ) | ( 1 << 0 );//set the priority level 7 and subpriority level 3
	IFS0CLR	= ( 1 << 16 );//clear the interrupt flag
	IEC0SET	= ( 1 << 16 );//enable timer 4 interrupts


	PORTF&=~(1<<12);//set the pwm output pin
	TRISF&=~(1<<12);


	trisMtrLeftDirClr	= ( 1 << bnMtrLeftDir );
	prtMtrLeftDirClr	= ( 1 << bnMtrLeftDir );	// forward
	
	// Configure right motor direction pin and set default direction.
	trisMtrRightDirClr	= ( 1 << bnMtrRightDir );	
	prtMtrRightDirSet	= ( 1 << bnMtrRightDir );	// forward

	// Configure Output Compare 2 to drive the left motor.

	OC2CON	= ( 1 << 2 ) | ( 1 << 1 );	// pwm set up
	OC2R	= dtcMtrStopped;
	OC2RS	= dtcMtrStopped;

	// Configure Output Compare 3.
	OC3CON = ( 1 << 3 ) | ( 1 << 2 ) | ( 1 << 1 );	// pwm
	OC3R	= dtcMtrStopped;
	OC3RS	= dtcMtrStopped;

	// Configure Timer 2.
	TMR2	= 0; // clear timer 2 count
	PR2		= 9999;//set the rate of the timer 3 interrupts

	// Configure Timer 3.
	TMR3	= 0;//clear timer 3 count
	PR3		= 9999;//set the rate of the timer 3 interrupts

	// Start timers and output compare units.
	T2CON		= ( 1 << 15 ) | ( 1 << TCKPS20 )|(1 << TCKPS21);
	OC2CONSET	= ( 1 << 15 );	// enable output compare module 2
	OC3CONSET	= ( 1 << 15 );	// enable output compare module 3
	T3CON		= ( 1 << 15 ) | ( 1 << TCKPS30 )|(1 << TCKPS31); ; 	// timer 3 prescale = 8

	// Configure Timer 5.
	TMR5	= 0;// clear timer 5 count
	PR5		=999; // period match every 1 ms
	IPC5SET	= ( 1 << 4 ) | ( 1 << 3 ) | ( 1 << 2 ) | ( 1 << 1 ) | ( 1 << 0 ); // set the priority level 7 and subpriority level 3
	IFS0CLR = ( 1 << 20);// clear the interrupt flag for timer 5
	IEC0SET	= ( 1 << 20);//enable the timer 5 interrupts
	
	// Start timers.
	T5CON = ( 1 << 15 ) | ( 1 << 5 ) | ( 1 << 4 ); // fTimer5 = fPb / 8
    
	//enable SPI
	SpiInit();

	// Enable multi-vector interrupts.
	INTEnableSystemMultiVectoredInt();
}






void Wait_ms(WORD delay) {

	WORD i;
 
	while(delay > 0){

		for( i = 0; i < 375; i ++){ 
			;;
		}
		delay -= 1;
	}
}

void i2c_wait(unsigned int cnt)
{
while(--cnt)
{
Nop();
Nop();
}
}

int Get0()
{
	int i,aux=0;
	int av[10]={0,0,0,0,0,0,0,0,0,0};
	for(i=0;i<=9;i++)
	{
		while(!AD1CON1bits.DONE);
		AD1CON1bits.DONE=0;
		av[i]=ADC1BUF0;
		if(i>=2) if(((av[i-1]>(av[i-2]+0x05))&&(av[i-1]>(av[i]+0x05)))||(((av[i-1]<(av[i-2]+0x05))&&(av[i-1]<(av[i]+0x05))))) av[i-1]=(av[i-2]+av[i])/2;

	}
	for(i=0;i<=9;i++) aux+=av[i];
	return aux/10;
}


int Get1()
{
	int i,aux=0;
	int av[10]={0,0,0,0,0,0,0,0,0,0};
	for(i=0;i<=9;i++)
	{
		while(!AD1CON1bits.DONE);
		AD1CON1bits.DONE=0;
		av[i]=ADC1BUF1;
		if(i>=2) if(((av[i-1]>(av[i-2]+0x05))&&(av[i-1]>(av[i]+0x05)))||(((av[i-1]<(av[i-2]+0x05))&&(av[i-1]<(av[i]+0x05))))) av[i-1]=(av[i-2]+av[i])/2;

	}
	for(i=0;i<=9;i++) aux+=av[i];
	return aux/10;
}

int Get2()
{
	int i,aux=0;
	int av[10]={0,0,0,0,0,0,0,0,0,0};
	for(i=0;i<=9;i++)
	{
		while(!AD1CON1bits.DONE);
		AD1CON1bits.DONE=0;
		av[i]=ADC1BUF2;
		if(i>=2) if(((av[i-1]>(av[i-2]+0x05))&&(av[i-1]>(av[i]+0x05)))||(((av[i-1]<(av[i-2]+0x05))&&(av[i-1]<(av[i]+0x05))))) av[i-1]=(av[i-2]+av[i])/2;

	}
	for(i=0;i<=9;i++) aux+=av[i];
	return aux/10;
}		
		
		
/************************************************************************/
