//Code based on the "Cerebot 32MX4 Servo Control Reference Design", provided by Digilent

#include <p32xxxx.h>
#include <plib.h>
#include <stdint.h>
#include "stdtypes.h"
#include "servo.h"


#define cntWdtInit 1600 	//nominal center of travel
#define cntWdtMin 600 		//left stop (counter-clockwise limit)
#define cntWdtMax 3200		//right stop (clockwise limit)

#define cntWdtDelta 10 //	This symbol specifies the amount to move the servo
						//	positions for the demo application. */

		
unsigned int	cntWdt1;


void	StartServoPulse (void);
void	EndServoPulse (void);
void	UpdateServoPos(void);
void 	ServoInit(void);



void ServoInit() {

	//Set up the i/o pins we are using for the servo channels
	//as outputs.
	prtServo1_6 &= ~((1 << bnServo1)|(1 << bnServo2)|(1 << bnServo2)|(1 << bnServo3)|\
						(1 << bnServo4)|(1 << bnServo5)|(1 << bnServo6));
	prtServo7_8 &= ~((1 << bnServo7)|(1 << bnServo8));

	trisServo1_6 &= ~((1 << bnServo1)|(1 << bnServo2)|(1 << bnServo2)|(1 << bnServo3)|\
						(1 << bnServo4)|(1 << bnServo5)|(1 << bnServo6));
	trisServo7_8 &= ~((1 << bnServo7)|(1 << bnServo8));

	cntWdt1 = cntWdtInit;

	// Set up the output compare module
	OC1CON	= ( 1 << 1 ) | ( 1 << 0 ); //Timer2 selected as clock source, events toggle OC2 pin
	OC1R	= cntWdtInit; //OC2R sets the pulse duration for the currently selected servo
	//Set up output compare interrupt
	IPC1SET = ( 1 << 20 ) | ( 1 << 19 ) | ( 1 << 18 ) | ( 1 << 17 ); //Priority level: 7, sub: 2
	IFS0CLR = ( 1 << 6 ); //clear output compare 2 interrupt flag
	IEC0SET = ( 1 << 6 ); //enable output compare 2 interrupt

	//Set up timer2 interrupt
	IPC2SET	= ( 1 << 4 ) | ( 1 << 3 ) | ( 1 << 2 ) | ( 1 << 0 );
	IFS0CLR	= ( 1 << 8 );
	IEC0SET	= ( 1 << 8 );

	OC1CONSET	= ( 1 << 15 );

	//Start the first servo pulse, interrupts will take over from there
	prtServo1 |= (1 << bnServo1);  //start beginning pulse of first servo.


}//end DeviceInit()



void StartServoPulse(void) {

	 
	
			prtServo1 |= (1 << bnServo1);
			OC1R = cntWdt1;  //load defined pulse width into compare register
	
}//end StartServoPulse function


void EndServoPulse(void) {

	
			prtServo1 &= ~(1 << bnServo1);
	

	
	
}  //end EndServoPulse function


void UpdateServoPos(void) {

	static unsigned int updateDelay;
	static BOOL left=fFalse;
	static BOOL right=fTrue;

		
		if(cntWdt1 == cntWdtMax)
		{
			cntWdt1 -= cntWdtDelta;
			left=fTrue;
			right=fFalse;
		}
		
		else if(cntWdt1 == cntWdtMin)
		{
			cntWdt1+=cntWdtDelta;
			left=fFalse;
			right=fTrue;
		}
	
		else {
		if(right)	cntWdt1 += cntWdtDelta;
		else		cntWdt1 -= cntWdtDelta;
		
		} 
		


}  //end UpdateServoPos function

